/*
 * GestureRecognizer.h
 *
 *  Created on: Jul 12, 2014
 *      Author: andrey-tsb
 */

#ifndef NUITRACK_GESTURE_RECOGNIZER_H_
#define NUITRACK_GESTURE_RECOGNIZER_H_

#include <functional>

#include "nuitrack/modules/Module.h"
#include "nuitrack/types/Export.h"
#include "nuitrack/types/GestureData.h"

namespace tdv
{
namespace nuitrack
{

/**
 * @brief Represent access to hand gesture recognition.
 * @ingroup GestureRecognizer_group
 */
class NUITRACK_API GestureRecognizer: public Module
{
public:
	typedef std::shared_ptr<GestureRecognizer> Ptr;

	/*
	 * GestureRecognizer new gestures event.
	 */
	typedef std::function<void (GestureData::Ptr)> OnNewGestures;

	/*
	 * On users state changes event
	 */
	typedef std::function<void (UserStateData::Ptr)> OnUserStateChange;

	/*
	 * On update event
	 */
	typedef std::function<void (UserGesturesStateData::Ptr)> OnUpdate;

	/**
	 * @brief Create GestureRecognizer.
	 *
	 * @return new instance GestureRecognizer.
	 */
	static GestureRecognizer::Ptr create();

	/**
	 * @brief Register OnNewGestures event callback.
	 *
	 * @param callback OnNewGestures callback
	 *
	 * @return callback handler
	 */
	virtual uint64_t connectOnNewGestures(const OnNewGestures& callback) = 0;

	/**
	 * @brief Unregister OnNewGestures event callback.
	 *
	 * @param callback handler
	 */
	virtual void disconnectOnNewGestures(uint64_t handler) = 0;

	/**
	 * @brief Register OnUsersStateChanges event callback.
	 *
	 * @param callback OnUsersStateChanges callback
	 *
	 * @return callback handler
	 */
	virtual uint64_t connectOnUserStateChange(const OnUserStateChange& callback) = 0;

	/**
	 * @brief Unregister OnUserStateChange event callback.
	 *
	 * @param callback handler
	 */
	virtual void disconnectOnUserStateChange(uint64_t handler) = 0;

	/**
	 * @brief Register OnUpdate event callback.
	 *
	 * @param OnUpdate callback
	 *
	 * @return callback handler
	 */
	virtual uint64_t connectOnUpdate(const OnUpdate& callback) = 0;

	/**
	 * @brief Unregister OnUpdate event callback.
	 *
	 * @param callback handler
	 */
	virtual void disconnectOnUpdate(uint64_t handler) = 0;

	/**
	 * @brief Get current processing time
	 *
	 * @return milliseconds
	 */
	virtual float getProcessingTime() const = 0;

	/**
	 * @brief Turn on/off control gestures (swipes/push)
	 *
	 * @param status (true - on, false - off)
	 */
	virtual void setControlGesturesStatus(bool status) = 0;
};

} /* namespace nuitrack */
} /* namespace tdv */

#endif /* NUITRACK_GESTURE_RECOGNIZER_H_ */
