/*
 * SkeletonTracker.h
 *
 *  Created on: Apr 14, 2014
 *      Author: Alex Desinov
 */

#ifndef NUITRACK_SKELETONTRACKER_H_
#define NUITRACK_SKELETONTRACKER_H_

#include <functional>

#include "nuitrack/modules/Module.h"
#include "nuitrack/types/Export.h"
#include "nuitrack/types/SkeletonData.h"

namespace tdv
{
namespace nuitrack
{

/**
 * @brief Represent access to user skeleton tracking.
 * @ingroup SkeletonTracker_group
 */
class NUITRACK_API SkeletonTracker: public Module
{
public:

	/**
	* Pointer to SkeletonTracker instance.
	*/
	typedef std::shared_ptr<SkeletonTracker> Ptr;

	/**
	 * SkeletonTracker update event.
	 * Event function receive smart pointer to SkeletonData, which is contain array of @ref Skeleton.
	 */
	typedef std::function<void (SkeletonData::Ptr)> OnUpdate;

	/**
	 * SkeletonTracker on new user event.
	 * When new user appears in the frame this callback fired. Fucnction should receive Id of the new user.
	 */
	typedef std::function<void (SkeletonTracker::Ptr, int)> OnNewUser;

	/**
	 * SkeletonTracker on lost user event.
	 * When user lost from the frame this callback fired. Fucnction should receive Id of the lost user.
	 */
	typedef std::function<void (SkeletonTracker::Ptr, int)> OnLostUser;

	/**
	 * @brief Create SkeletonTracker.
	 *
	 * @return New SkeletonTracker instance.
	 */
	static SkeletonTracker::Ptr create();

	/**
	 * @brief Register OnUpdate event callback.
	 *
	 * @param[in] callback OnUpdate event callback.
	 *
	 * @return callback Callback handler.
	 */
	virtual uint64_t connectOnUpdate(const OnUpdate& callback) = 0;

	/**
	 * @brief Unregister OnUpdate event callback.
	 *
	 * @param[in] callback Callback handler.
	 */
	virtual void disconnectOnUpdate(uint64_t handler) = 0;

	/**
	 * @brief Register OnNewUser event callback.
	 *
	 * @param[in] callback OnNewUser event callback.
	 *
	 * @return callback Callback handler.
	 */
	virtual uint64_t connectOnNewUser(const OnNewUser& callback) = 0;

	/**
	 * @brief Unregister OnNewUser event callback.
	 *
	 * @param[in] callback Callback handler.
	 */
	virtual void disconnectOnNewUser(uint64_t handler) = 0;

	/**
	 * @brief Register OnLostUser event callback.
	 *
	 * @param[in] callback OnLostUser event callback.
	 *
	 * @return callback Callback handler.
	 */
	virtual uint64_t connectOnLostUser(const OnLostUser& callback) = 0;

	/**
	 * @brief Unregister OnLostUser event callback.
	 *
	 * @param[in] callback Callback handler.
	 */
	virtual void disconnectOnLostUser(uint64_t handler) = 0;

	/**
	 * @brief Set maximum number of users for tracking.
	 *
	 * @param[in] numUsers Maximum users number.
	 */
	virtual void setNumActiveUsers(int numUsers) = 0;

	/**
	 * @brief Get users skeleton auto tracking status.
	 *
	 * @return Is auto tracking enabled status.
	 */
	virtual bool isAutoTracking() const = 0;

	/**
	 * @brief Set users skeleton auto tracking.
	 *
	 * @param[in] tracking Enable or disable auto tracking.
	 */
	virtual void setAutoTracking(bool tracking) = 0;

	/**
	 * @brief Start tracking user by ID.
	 *
	 * @param[in] userID Id of user which tracking should be started.
	 */
	virtual void startTracking(int userID) = 0;

	/**
	 * @brief Stop tracking user by ID.
	 *
	 * @param[in] userID Id of user which tracking should be stopped.
	 */
	virtual void stopTracking(int userID) = 0;

	/**
	 * @brief Check tracking status of user by ID.
	 *
	 * @return Tracking status of user.
	 */
	virtual bool isTracking(int userID) = 0;

	/**
	 * @brief Get smart pointer to last avilable UserSkeletons.
	 *
	 * @return Smart pointer to last avilable UserSkeletons.
	 */
	virtual SkeletonData::Ptr getSkeletons() const = 0;

	/**
	 * @brief Get current processing time.
	 *
	 * @return Frame processing time in milliseconds.
	 */
	virtual float getProcessingTime() const = 0;
};

} /* namespace nuitrack */
} /* namespace tdv */

#endif /* NUITRACK_SKELETONTRACKER_H_ */
