/*
 * DepthSensor.h
 *
 *  Created on: Apr 4, 2014
 *      Author: Alex Desinov
 */

#ifndef NUITRACK_DEPTHSENSOR_H_
#define NUITRACK_DEPTHSENSOR_H_

#include <vector>
#include <functional>

#include "nuitrack/modules/Module.h"
#include "nuitrack/types/Export.h"
#include "nuitrack/types/DepthFrame.h"
#include "nuitrack/types/Vector3.h"
#include "nuitrack/types/OutputMode.h"

namespace tdv
{
namespace nuitrack
{

/**
 * @brief Represent access to sensor device.
 * @ingroup DepthSensor_group
 */
class NUITRACK_API DepthSensor: public Module
{
public:

	/**
	* Pointer to DepthSensor instance.
	*/
	typedef std::shared_ptr<DepthSensor> Ptr;

	/**
	 * DepthSensor new frame event.
	 * Event function receive smart pointer to DepthFrame, which is contain depth map.
	 */
	typedef std::function<void (DepthFrame::Ptr)> OnNewFrame;

	/**
	 * @brief Create DepthSensor.
	 *
	 * @return New DepthSensor instance.
	 */
	static DepthSensor::Ptr create();

	/**
	 * @brief Get output mode for DepthSensor.
	 *
	 * Returns resolution, fps, fov and other sensor information.
	 *
	 * @return Output mode.
	 */
	virtual const OutputMode& getOutputMode() const = 0;

	/**
	 * @brief Get mirror mode of DepthSensor.
	 *
	 * @return Mirror mode of DepthSensor.
	 */
	virtual bool isMirror() const = 0;

	/**
	 * @brief Set mirror mode for DepthSensor.
	 *
	 * @param[in] mirror Mirror mode for DepthSensor.
	 */
	virtual void setMirror(bool mirror) = 0;

	/**
	 * @brief Register OnNewFrame event callback.
	 *
	 * @param[in] callback OnNewFrame event callback.
	 *
	 * @return Callback handler.
	 */
	virtual uint64_t connectOnNewFrame(const OnNewFrame& callback) = 0;

	/**
	 * @brief Unregister OnNewFrame event callback.
	 *
	 * @param[in] callback Callback handler.
	 */
	virtual void disconnectOnNewFrame(uint64_t handler) = 0;

	/**
	 * @brief Get smart pointer to last avilable DepthFrame.
	 *
	 * @return Smart pointer to last avilable DepthFrame.
	 */
	virtual DepthFrame::Ptr getDepthFrame() const = 0;

	/**
	 * @brief Convert projective coordinates to real world coordinates from Vector3 point.
	 * @param[in] p Projective point coordinates.
	 * @return Converted real world point.
	 */
	virtual Vector3 convertProjToRealCoords(const Vector3& p) const = 0;

	/**
	* @brief Convert projective coordinates to real world coordinates from x, y and depth map.
	* @param[in] x,y Projective point coordinates.
	* @param[in] depth Depth map.
	* @return Converted real world point.
	*/
	virtual Vector3 convertProjToRealCoords(size_t x, size_t y, DepthFrame::DataType depth) const = 0;

	/**
	 * @brief Convert real world coordinates to projective coordinates from Vector3 point.
	 * @param[in] p Real world point coordinates.
	 * @return Converted projective point.
	 */
	virtual Vector3 convertRealToProjCoords(const Vector3& p) const = 0;

	/**
	* @brief Convert real world coordinates to projective coordinates from x, y, z.
	* @param[in] x,y,z Real world point coordinates.
	* @return Converted projective point.
	*/
	virtual Vector3 convertRealToProjCoords(float x, float y, float z) const = 0;
};

} /* namespace nuitrack */
} /* namespace tdv */

#endif /* NUITRACK_DEPTHSENSOR_H_ */
